"""
Infra bootstrap

"""
import argparse
import os
import sys
import logging
import boto3

default_ec2 = boto3.client('ec2')
enabled_regions = set(r['RegionName'] for r in default_ec2.describe_regions()['Regions'])

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)
stream_handler = logging.StreamHandler()
stream_handler.setLevel(logging.INFO)
logger.addHandler(stream_handler)
check_argument = True

def main(action, options, group, env, region, provider):
    """
    main
    """

    config_dir = "./../../../configs/{}/{}/terraform".format(group, env)
    bootstrap_dir = "./terraform/bootstrap/{}".format(provider)
    command_init = "terraform init"
    command_apply = "terraform {} {} -var=group={}\
    -var=env={} -var=region={key} -state={}/bootstrap-{}-{key}.tfstate".format(
        action, options, group, env, config_dir, provider, key=region)
    os.chdir(bootstrap_dir)
    os.system(command_init)
    os.system(command_apply)

if __name__ == '__main__':
    # Initialize the parser
    parser = argparse.ArgumentParser(description="Boot Strap script ")
    # Add the parameters positional/optional
    parser.add_argument('--provider', help="provider aws", default="aws")
    parser.add_argument('--account', help="account <group>-<env>")
    parser.add_argument('--action', help="plan apply or destroy", default="apply")
    parser.add_argument('--region', help="eu-west-1 by default", default="eu-west-1")
    parser.add_argument('--approve', help="Auto-approve option ,set 'yes' to enable it")
    # Parse the arguments
    args = parser.parse_args()
    act = args.action
    reg = args.region
    opt = ""
    if args.provider == 'aws' or args.provider == 'gcp':
        pro = args.provider
    else:
        logger.error("Error in provider argument")
        sys.exit()
    if args.account is None:
        logger.error("Missing account argument,Wedeployer Can not run without Account")
        sys.exit(1)
    try:
        account_name = args.account
        account_name = account_name.split("-")
        grp = account_name[0]
        environment = account_name[1]
    except IndexError:
        logger.error("Error in syntax argument")
        sys.exit()
    except AttributeError:
        logger.error("Attribute Error")

    if act not in ("apply", "plan", "destroy"):
        logger.error("Error ,Bad Action for Terraform,just 'apply','plan' and 'destroy' are available")
        check_argument = False

    if reg not in enabled_regions:
        logger.error("No Region Found with this Name")
        check_argument = False
    if args.approve == "yes":
        opt = "-auto-approve"
    elif args.approve is not None:
        logger.error("Error , Auto-Approve is disabled by default ,it takes only 'yes'")

    if check_argument is True:
        main(act, opt, grp, environment, reg, pro)
    else:
        logger.info("You must fix errors mentioned above")
        sys.exit(1)



"""
genrate configuration files
"""
#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import logging
import sys
import os
import argparse
from subprocess import check_output, STDOUT
from jinja2 import Environment, FileSystemLoader

logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)
stream_handler = logging.StreamHandler()
stream_handler.setLevel(logging.INFO)
logger.addHandler(stream_handler)

def get(nom, region, whichip):
    """
    get method

    """
    ip = 'aws autoscaling describe-auto-scaling-instances --region {region} \
    --output text --query "AutoScalingInstances[?AutoScalingGroupName==\'{}\'].InstanceId "\
    | xargs -n1 aws ec2 describe-instances --instance-ids $ID --region {region} \
    --query Reservations[].Instances[].{}IpAddress --output text'.format(
        nom, whichip, region=region)

    output = check_output(ip, shell=True, stderr=STDOUT)
    ips = output.decode("utf-8")
    return ips


def main(group, env):
    """
    Main

    """
    bastion_public_ip = get("{}-{}-bastion-asg".format(group, env), "eu-west-1", "Public")
    workdir = os.getcwd()
    tpl_dir = Environment(loader=FileSystemLoader('{}/templates/'.format(workdir)))
    template_cfg = tpl_dir.get_template('ansible-cfg.j2')
    template_ssh = tpl_dir.get_template('ssh-cfg.j2')
    template_in = tpl_dir.get_template('inventory.j2')
    nginx_ips = get("{}-{}-nginx-asg".format(group, env), "eu-west-1", "Private")
    nginx_ips = nginx_ips.split("\n")

    if nginx_ips == ['']:
        logger.error("Nginx IPs not found, verify your auto-scaling group ")

    if bastion_public_ip == "":
        logger.error("Bastion Ip not found , verify your auto-scaling group ")
        sys.exit(1)

    tpl_cfg_output = template_cfg.render(group=group, env=env)
    tpl_ssh_output = template_ssh.render(group=group, env=env, bastion_public_ip=bastion_public_ip, nginx_ips=nginx_ips)
    tpl_invt_output = template_in.render(group=group, env=env, bastion_public_ip=bastion_public_ip, nginx_ips=nginx_ips)

    try:
        with open('./{}-{}-ansible.cfg'.format(group, env), "w") as f:
            f.write(tpl_cfg_output)
            logger.info("ansible.cfg generated")

    except FileNotFoundError:
        logger.error("Failed to save file")
        sys.exit(1)

    try:
        with open('./configs/{}/{}/ansible/ssh.cfg'.format(group, env), "w") as f:
            f.write(tpl_ssh_output)
            logger.info("ssh file generated")
    except FileNotFoundError:
        logger.error("Failed to save file")
        sys.exit(1)

    try:
        with open('./configs/{}/{}/ansible/inventory'.format(group, env), "w") as f:
            f.write(tpl_invt_output)
            logger.info("inventory generated")
    except FileNotFoundError:
        logger.error("Failed to save file")
        sys.exit(1)
if __name__ == '__main__':
    parser = argparse.ArgumentParser(description=" get ips")
    parser.add_argument('--group', help="group")
    parser.add_argument('--env', help="env")
    args = parser.parse_args()
    if args.group is None or args.env is None:
        logger.error("Missing argument , please verify help for more informations")
        sys.exit(1)
    try:
        grp = str(args.group)
        environment = str(args.env)

    except NameError:
        sys.exit(1)
    except IndexError:
        sys.exit(1)
    main(grp, environment)
